import { getJSONfromString } from "@/src/helpers/stringHelpers";
import { InventoryDocumentProps } from "@/src/models/inventoryModels/inventoryModel";
import {
    addEquipment,
    addItem,
    addPowerSuit,
    combineInventoryChanges,
    getInventory,
    updateSlots
} from "@/src/services/inventoryService";
import { getAccountIdForRequest } from "@/src/services/loginService";
import { IEquipmentClient } from "@/src/types/inventoryTypes/commonInventoryTypes";
import {
    IInventoryClient,
    IInventoryDatabase,
    InventorySlot,
    TEquipmentKey
} from "@/src/types/inventoryTypes/inventoryTypes";
import { IInventoryChanges } from "@/src/types/purchaseTypes";
import { RequestHandler } from "express";
import { HydratedDocument } from "mongoose";

export const giveStartingGearController: RequestHandler = async (req, res) => {
    const accountId = await getAccountIdForRequest(req);
    const startingGear = getJSONfromString<Partial<IInventoryClient>>(String(req.body));
    const inventory = await getInventory(accountId);

    const inventoryChanges = await addStartingGear(inventory, startingGear);

    res.send(inventoryChanges);
};

//TODO: RawUpgrades might need to return a LastAdded
const awakeningRewards = [
    "/Lotus/Types/StoreItems/AvatarImages/AvatarImageItem1",
    "/Lotus/Types/StoreItems/AvatarImages/AvatarImageItem2",
    "/Lotus/Types/StoreItems/AvatarImages/AvatarImageItem3",
    "/Lotus/Types/StoreItems/AvatarImages/AvatarImageItem4",
    "/Lotus/Types/Restoratives/LisetAutoHack",
    "/Lotus/Upgrades/Mods/Warframe/AvatarShieldMaxMod"
];

type test = IInventoryClient &  [key in TEquipmentKey]?: Partial<IEquipmentClient>[] ;
export const addStartingGear = async (
    inventory: HydratedDocument<IInventoryDatabase, InventoryDocumentProps>,
    startingGear: test,
    _skipTutorial: boolean = false
) => {
    const { LongGuns, Pistols, Suits, Melee } = startingGear;

    //TODO: properly merge weapon bin changes
    const inventoryChanges: IInventoryChanges = {};
    addEquipment(inventory, "LongGuns", LongGuns[0].ItemType, undefined, inventoryChanges);
    addEquipment(inventory, "Pistols", Pistols[0].ItemType, undefined, inventoryChanges);
    addEquipment(inventory, "Melee", Melee[0].ItemType, undefined, inventoryChanges);
    addEquipment(inventory, "Suits", Suits[0].ItemType, undefined, inventoryChanges, { Configs: Suits[0].Configs });
    addEquipment(
        inventory,
        "DataKnives",
        "/Lotus/Weapons/Tenno/HackingDevices/TnHackingDevice/TnHackingDeviceWeapon",
        undefined,
        inventoryChanges,
        { XP: 450_000 }
    );
    addEquipment(
        inventory,
        "Scoops",
        "/Lotus/Weapons/Tenno/Speedball/SpeedballWeaponTest",
        undefined,
        inventoryChanges
    );

    updateSlots(inventory, InventorySlot.SUITS, 0, 1);
    updateSlots(inventory, InventorySlot.WEAPONS, 0, 3);
    inventoryChanges.SuitBin = { count: 1, platinum: 0, Slots: -1 };
    inventoryChanges.WeaponBin = { count: 3, platinum: 0, Slots: -3 };

    await addItem(inventory, "/Lotus/Types/Keys/VorsPrize/VorsPrizeQuestKeyChain");
    inventory.ActiveQuest = "/Lotus/Types/Keys/VorsPrize/VorsPrizeQuestKeyChain";

    inventory.PremiumCredits = 50;
    inventory.PremiumCreditsFree = 50;
    inventoryChanges.PremiumCredits = 50;
    inventoryChanges.PremiumCreditsFree = 50;
    inventory.RegularCredits = 3000;
    inventoryChanges.RegularCredits = 3000;

    for (const item of awakeningRewards) {
        const inventoryDelta = await addItem(inventory, item);
        combineInventoryChanges(inventoryChanges, inventoryDelta.InventoryChanges);
    }
    // currentloadoutids

    //     "LoadOutPresets": {
    //     "NORMAL": [
    //       {
    //         "s": {
    //           "ItemId": { "$oid": "67b73fbfb56b4b84d70433c2" },
    //           "mod": 0,
    //           "cus": 0
    //         },
    //         "p": {
    //           "ItemId": { "$oid": "67b73fbfb56b4b84d70433c5" },
    //           "mod": 0,
    //           "cus": 0
    //         },
    //         "l": {
    //           "ItemId": { "$oid": "67b73fbfb56b4b84d70433c4" },
    //           "mod": 0,
    //           "cus": 0
    //         },
    //         "m": {
    //           "ItemId": { "$oid": "67b73fbfb56b4b84d70433c6" },
    //           "mod": 0,
    //           "cus": 0
    //         },
    //         "ItemId": { "$oid": "67b73fbfb56b4b84d70433c7" }
    //       }
    //     ]
    //   },

    inventory.PlayedParkourTutorial = true;
    inventory.ReceivedStartingGear = true;

    //originally ship is returned
    //also weaponskins
    return inventoryChanges;
};
